@@ -21,6 +21,37 @@
 #    Comments to: arsperl@smurfland.cit.buffalo.edu
 #
 # $Log: ARS.pm,v $
+# Revision 1.25  1997/11/26 20:06:48  jcmurphy
+# 1.53
+#
+# Revision 1.24  1997/11/10 23:50:36  jcmurphy
+# 1.5206
+#
+# Revision 1.23  1997/11/04 18:19:09  jcmurphy
+# version # update
+#
+# Revision 1.22  1997/10/29 21:57:34  jcmurphy
+# version number change
+#
+# Revision 1.21  1997/10/29 21:54:07  jcmurphy
+# 1.5204
+#
+# Revision 1.20  1997/10/20 21:00:05  jcmurphy
+# 1.5203 beta. code cleanup. winnt additions. malloc/free
+# debugging code.
+#
+# Revision 1.19  1997/10/13 12:26:29  jcmurphy
+# 1.5202
+#
+# Revision 1.18  1997/10/09 15:21:33  jcmurphy
+# 1.5201 version
+#
+# Revision 1.17  1997/10/09 00:57:56  jcmurphy
+# version update. added ARS::DEBUGGING
+#
+# Revision 1.16  1997/10/09 00:49:46  jcmurphy
+# 1.52: uninit'd var bug fix
+#
 # Revision 1.15  1997/10/07 14:29:01  jcmurphy
 # *** empty log message ***
 #
@@ -75,12 +106,23 @@ sub FETCH {
 		    4 => "INTERNAL ERROR",
 		   -1 => "TRACEBACK");
     for($i = 0; $i < $ARS::ars_errhash{numItems}; $i++) {
-	$s .= sprintf("[%s] %s (ARERR \#%d)",
-		      $mTypes{@{$ARS::ars_errhash{messageType}}[$i]},
-		      @{$ARS::ars_errhash{messageText}}[$i],
-		      @{$ARS::ars_errhash{messageNum}}[$i]);
-	if($i < $ARS::ars_errhash{numItems}-1) {
-	    $s .= "\n";
+
+	# If debugging is not enabled, don't show traceback messages
+
+	if($ARS::DEBUGGING == 1) {
+	    $s .= sprintf("[%s] %s (ARERR \#%d)",
+			  $mTypes{@{$ARS::ars_errhash{messageType}}[$i]},
+			  @{$ARS::ars_errhash{messageText}}[$i],
+			  @{$ARS::ars_errhash{messageNum}}[$i]);
+	    $s .= "\n" if($i < $ARS::ars_errhash{numItems}-1);
+	} else {
+	    if(@{$ARS::ars_errhash{messageType}}[$i] != -1) {
+		$s .= sprintf("[%s] %s (ARERR \#%d)",
+			      $mTypes{@{$ARS::ars_errhash{messageType}}[$i]},
+			      @{$ARS::ars_errhash{messageText}}[$i],
+			      @{$ARS::ars_errhash{messageNum}}[$i]);
+		$s .= "\n" if($i < $ARS::ars_errhash{numItems}-1);
+	    }
 	}
     }
     return $s;
@@ -115,10 +157,12 @@ ars_GetAdminExtension ars_GetEscalation ars_GetFullTextInfo
 ars_GetListGroup ars_GetListSQL ars_GetListUser
 ars_GetListVUI ars_GetServerInfo
 ars_CreateActiveLink ars_CreateAdminExtension
+ars_GetControlStructFields ars_GetVUI
 $ars_errstr %ARServerStats %ars_errhash
 );
 
-$VERSION = '1.51';
+$VERSION   = '1.53';
+$DEBUGGING = 0;
 
 bootstrap ARS $VERSION;
 tie $ars_errstr, ARS::ERRORSTR;
